;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; e2modled.asm -- Example TI-86 program showing how to use	;
; Expander II's 6 pin module port to flash LEDs :)		;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#include "asm86.h"
#include "ti86asm.inc"

; Start of program
.org _asm_exec_ram
	ld a, $C0		; Set W1 and R1
        out (7),a		; Setup link port

	ld (spbak),sp		; Backup stack

	ld bc,100		;
	ld (speed),bc		;

	call BUSY_OFF		; Turn off stupid runindic
	call CLEARLCD		; Clear screen

	ld b,%00111111		;
	call set_mod_dir	; Setup mod port direction
	ld a,1			;

led_loop:
	ld b,a			;
	call set_mod_port	;
	ld a,b			;
	push af			;
	push af
	ld hl,$0000		;
	ld (_penCol),hl		; Wipe space
	ld hl,space
	call _vputs		; Overwrite...
	ld hl,$0035		;
	ld (_penCol),hl		;
	ld hl,space		;
	call _vputs		;
	ld hl,$0035		;
	ld (_penCol),hl		;
	ld hl,(speed)		;
	call vputHL		; Display
	pop af			;
	call disp_byte		; Display byte in binary
	call wait_loop		;
	pop af
	rla			;
	cp $40
	jr z,fix_it 
	jr led_loop		;

fix_it:
	ld a,1			;
	jr led_loop
	

;input_loop:
;	call read_mod_port	; Read module port
;	ld c,a			;
;	ld a,(oldin)		;
;	cp c			; Same as c?
;	jr nz,update_disp	; Update display
;	call GET_KEY		; Get key
;	cp K_EXIT		;
;	jr z,exitnopause	;
;	call wait_loop		; Wait, to keep from screwing up port
;	call wait_key		; Wait for user to update...
;	jr input_loop		; Repeat...

;update_disp:
;	ld a,c			; Put input back into a
;	ld (oldin),a		; Update...
;	push af			; Save a
;	ld hl,0000		;
;	ld (_penCol),hl		; Wipe space
;	ld hl,space
;	call _vputs		; Overwrite...
;	pop af
;	call disp_byte		; Display byte in binary
;	jr input_loop		; Keep going..

; Displays a in "binary", MSB first
disp_byte:
	ld b,8			; Display 8 bits
	ld hl,buffer		;
disp_loop:
	rla			; Rotate a	
	jr c,disp_one		;
	ld (hl),48		; "0"
	jr next_bit		;
disp_one:
	ld (hl),49		; "1"
next_bit:			;
	inc hl
	djnz disp_loop		;
	inc hl			;
	ld (hl),0		; Null terminated
	ld hl,0000		; Could be optimized, but fuck it
	ld (_penCol),hl		;
	ld hl,buffer
	call _vputs		; Finally, display
	ret			; ... and return

wait_loop:
	ld bc,(speed)		; Blah
loop_dec:
	push bc
	call GET_KEY		;
	cp K_EXIT		;
	jr z,exitnopause	;
	cp K_UP			;
	jr z,decspeed		;	
	cp K_DOWN		;
	jr z,incspeed		;
continue_loop:
	pop bc			;
	dec bc			;
	ld a,b			;
	or c			;
	jr nz,loop_dec		;
	ret			;

incspeed:
	ld hl,(speed)		;
	ld de,10
	add hl,de		;
	ld (speed),hl		;
	jr continue_loop	;

decspeed:
	ld a,l			;
	cp 10			;
	jr z,check_h		;
dec_now:
	ld hl,(speed)		;
	ld de,-10		;
	add hl,de		;
	ld (speed),hl		;
	jr continue_loop	;

check_h:
	ld a,h			;
	or a			;
	jr z,continue_loop	;
	jr dec_now		;

exitpause:
	call wait_key		; Return
exitnopause:
	call CLEARLCD		;
	ld sp,(spbak)		;
	ret			;

wait_key:
	call GET_KEY		; ROM keyscan call
	cp K_NOKEY		;
	jr z,wait_key		;
	ret			;

; hl to decimal display routine...
; destroys de
vputHL:				;
	ld de,buffer+6		;
	xor a			; Clear last char
	ld (de),a		;
	ld b,5			; Only display five characters..
RepUnp:				;
	call UNPACK_HL		;
	or a			; If next digit is a 0, print so far
	jr z,check_rest		;
not_zero:
	dec de			;
	add a,48		;
	ld (de),a		;
	djnz RepUnp		;
disp_it:
	ex de,hl		;
	jp D_ZM_STR		;

; Checks to see if hl is at 00, if it is, go right to disp_it
; If its not, then we just have a normal 0 digit...
check_rest:
	push af
	ld a,h			;
	or l			;
	jr nz,disp_nz		; We're done...
	inc sp			;
	inc sp			;
	jr disp_it		;
disp_nz:
	pop af
	jr not_zero		; Otherwise, keep going

to:	.db	"Time Out!",0	; Needed by modlink.asm
space:	.db	"             ",0	; Overwrite old...

spbak	equ	TEXT_MEM	; Stack backup for modlink.asm
oldin	equ	TEXT_MEM+2	; Old input... don't over-refresh
speed	equ	TEXT_MEM+3
buffer	equ	TEXT_MEM+5	; 9 bytes, no _vputc

; E2 link routines
#include "modlink.asm"

.end				; End of file
