port		equ	$07

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sets module port pin directions... 1 is output, 0 is input	;
; desired dir should be in b register				;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
set_mod_dir:
	ld a,$22		;
	jr send_mod_port	;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Writes b0-b5 of register b to E2 module port			;
; b6 (busy LED) is masked out by firmware			;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
set_mod_port:
	ld a,$21		;

send_mod_port:			;
	di			;
	call send_byte		;
	ld a,b			; send_byte push/pop's b so should be ok
	call send_byte		;
	ei			; Re-enable interrupts
	ret			; Return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Requests status of memory chip -- usually 1000000 on all the	;
; chips I've tested... does NOT follow AT45D041 datasheet :(	;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
stat_req:
	ld a,$57		; This is the status op code
	jr send_rec_ret		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Reads module port... 6 port lines occupy b0-b5		;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
read_mod_port:
	ld a,$20		; Read module port op code
	jr send_rec_ret		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Gets the E2/EuP's firmware revision				;
; =< 4 is an EuP, >= 5 is an E2					;
; Doesn't matter though, both work same way... only E2 has	;
; a module port however						;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
ver_get:
	ld a,$FF		; Status get op code

; Just to save some code..
send_rec_ret:			;
	di			;
	call send_byte		;
	call rec_byte		;
	ei			;
	ret			;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Receives byte using TI protocol				;
; Returns with z set if errored out.. otherwise byte = a	;
; IT IS CRUCIAL THAT THE STACK IS CLEAN WHEN THIS IS CALLED	;
; BECAUSE IF IT ERRORS OUT IT'LL DO -TWO- RETs TO RETURN TO	;
; HIGHEST LEVEL API CALL					;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
rec_byte:			;
	push bc			; Back 'em up... destroyed
	push de			;
	ld b,$08		; 8 bits..

r0:
	ld de,$FFFF		;
;	jr r2			;

r1:
	in a,(port)		;
	and $03			; Both low?
	jr z,linkerr		;  That's an instant error
	cp $03			; Both high?
	jr nz,r3		;  No... go to r3

;	in a,(port)		;
;	and $03			;
;	jr z,linkerr		;
;	cp $03			;
;	jr nz,r3		;
r2:
	dec de			; Decrease counter...
	ld a,d			; Is de at 0 yet?
	or e			;
	jr nz,r1		;
	jr linkerr		; Yes it is, return

r3:
	sub $02			; Which line is active?
	jr nc,r8		; Line 2 (bit 1)
	ld a,$D4		; Bit 0...
	out (port),a		;
	rr c			;
	ld de,$FFFF		;
;	nop
;	nop
;	nop
;	nop
r4:
	in a,(port)		;
	and $03			; Clear out everything but first two
	cp $02			; Line 1 clear?
	jr z,r5			;
	dec de			; Line 2 is clear...
	ld a,d			;
	or e			;
	jr nz,r4		;
	jr linkerr		; Timeout...
r5:
	ld a,$C0		;
	out (port),a		;
	ld d,$04		;
;	nop
;	nop
;	nop
;	nop
r6:
	dec d			;
	jr z,r7			;
	in a,(port)		;
	and $03			;
	cp $03			;
	jr nz,r6		;
r7:
	djnz r0			;
	ld a,c			;
	jr linkok		;

r8:
	ld a,$E8		;
	out (port),a		;
	rr c			;
	ld de,$FFFF		;
;	nop
;	nop
;	nop
;	nop
r9:
	in a,(port)		;
	and $03			;
	cp $01			;
	jr z,r5			;
	dec de			;
	ld a,d			;
	or e			;
	jr nz,r9		; Errored out...

linkerr:			; Cheap stack tricks, but it works..
	ld sp,(spbak)		; Backup stack pointer... save us
	ld hl,$3030		; Setup for number display
	ld (_penCol),hl		;
	ld hl,to		; Display
	call D_ZM_STR		; Display time out string...
	jp exitpause		; Return to main menu

linkok:
	pop de			; Just get vars back and return
	pop bc			;
	ret			;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sends byte out using TI protocol				;
; Input: a							;
; Returns with z set if errored out...				;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
send_byte:
	push bc			; Back up destroyed registers
	push de			;
	ld c,a
	ld b,$08		;
s0:
	ld de,$FFFF		;
	rr c			;
	jr nc,s1		; Send a 0
	ld a,$E8		; %11101000 -- Activate red wire
	jr s2			;
s1:
	ld a,$D4		; %11010100 -- Activate white wire
s2:
	out (port),a		;
;	nop
;	nop
;	nop
;	nop
s3:
	in a,(port)		;
	and $03			; Both low?
	jr z,s4			;

;	in a,(port)		;
;	and $03			;
;	jr z,s4			;

	dec de			;
	ld a,d			;
	or e			;
	jr nz,s3		;
	jr linkerr		; We've errored out
s4:
	ld a,$C0		; %110000000 -- Activate both wires?
	out (port),a		;
	ld de,$FFFF		;
;	nop
;	nop
;	nop
;	nop
s5:
	dec de			;
	ld a,d			;
	or e			;
	jr z,linkerr		;
	in a,(port)		;
	and $03			;
	cp $03			; Both high?
	jr nz,s5		;
	djnz s0			;
	jr linkok

